/* Player class that uses an embedded JESS Rete engine and rule base
 * to decide which move to make */

package wclient;

import java.net.*;
import wInput.InputTaker;
import wAction.Action;
import wPercept.TransferPercept;
import java.awt.*;
import jess.*;
import java.io.*;

public class Player
{
  int agentID;
  Communicator pipe;
  Rete engine;


  public Player(int id, Communicator comm) 
	throws JessException
  {
    agentID = id;
    System.out.println("I am Player Number " + agentID);
    pipe = comm;

    // Create a JESS Rete engine, load the rules, and (reset).

    engine = new Rete();
    try{
	engine.executeCommand("(batch JessClient.clp)");
    } catch(JessException e) {
	System.out.println("Exception thrown when loading JessClient.clp");
    }
  }

  public int process(TransferPercept tp, Frame1 frame, Frame2 frame2)
	throws JessException 
  {
    // Instructions for Intelligent Agents Design
    // Do not Remove Section I code if want to display GUI and scores
    // Section I Start
    pipe.checkAvailability();
    this.statusDisplay(tp);
    frame.updateDisplay(tp);
    frame2.makeImage(tp);

    // Section I End.

    // Start of the code for the Intelligent Agent.

    // Create the Percept Fact object and set its slots from the
    // TransferPercept

    Fact f = new Fact("Percepts", engine);
    f.setSlotValue("glitter", new Value(tp.getGlitter()));
    f.setSlotValue("stench", new Value(tp.getStench()));
    f.setSlotValue("breeze", new Value(tp.getBreeze()));
    f.setSlotValue("bump", new Value(tp.getBump()));
    f.setSlotValue("scream", new Value(tp.getScream()));
    f.setSlotValue("numArrow", new Value(tp.getNumArrow(), RU.INTEGER));
    f.setSlotValue("actionTaken", new Value(tp.getActionTaken(), RU.INTEGER));
    f.setSlotValue("hint", new Value(tp.getHint(), RU.STRING));
    f.setSlotValue("status", new Value(tp.getStatus(), RU.INTEGER));
    f.setSlotValue("score", new Value(tp.getScore(), RU.INTEGER));
    f.setSlotValue("numGold", new Value(tp.getNumGold(), RU.INTEGER));

    // Assert the Fact, run the JESS client, and retract the Fact
    // (because it will go away at the end of this method).
    engine.assertFact(f);
    engine.run();
    engine.retract(f);

    // The JESS client stores the move in the hash table with the key
    // "MOVE"; we retrieve it, convert it to an int, and return that
    // int as our move.
    Value v = engine.fetch( "MOVE" );
    return v.intValue( engine.getGlobalContext() );
  }

  public void statusDisplay(TransferPercept tp)
  {
    Action acted = new Action();
    System.out.println("____________________________________________");
    System.out.println(" STATUS INFORMATION");
    System.out.println("____________________________________________");
    System.out.println("Last Action Executed: " + acted.printAction(tp.getActionTaken()));
    System.out.println("My Status is: " + tp.getStatus());
    System.out.println("My Score is: " + tp.getScore());
    System.out.println("Arrows: " + tp.getNumArrow());
    System.out.println("Gold Pieces: " + tp.getNumGold());
    System.out.println("");
    System.out.println("____________________________________________");
    System.out.println(" PERCEPT INFORMATION");
    System.out.println("____________________________________________");
    System.out.println("Stench: " + tp.getStench());
    System.out.println("Breeze: " + tp.getBreeze());
    System.out.println("Glitter: " + tp.getGlitter());
    System.out.println("Bump: " + tp.getBump());
    System.out.println("Scream: " + tp.getScream());
    System.out.println("____________________________________________");
    System.out.println("HINT INFORMATION");
    System.out.println("hint: " + tp.getHint());
    System.out.println("____________________________________________");
  }

}
