package wPercept;

import java.io.*;
import java.util.*;

/**
Class whose objects are communicated between the server and clients.
Defines the various percepts, status and scores for the client and
provides methods to access the information with.
*/

public class TransferPercept implements Serializable
{
   private boolean stench;
   private boolean breeze;
   private boolean glitter;
   private boolean bump;
   private boolean scream;
   private int actionTaken;
   private int score;
   private double averageScore;
   private double stdDeviation;
   private int numArrow;
   private int numGold;
   private int status;
   private int[][][] imagePixels;
   private int imageHeight;
   private int imageWidth;
   private String hint;

   public TransferPercept()
   {
    stench = breeze = glitter = bump = scream = false;
   }

   public void setPercept(boolean st, boolean br, boolean gli, boolean bu, boolean sc)
   {
    stench = st;
    breeze = br;
    glitter = gli;
    bump = bu;
    scream = sc;
   }


   public void setActionTaken(int a)
   {
    actionTaken = a;
   }

   public void setScore(int a)
   {
    score = a;
   }

   public int getScore()
   {
    return score;
   }

   public void setNumArrow(int a)
   {
    numArrow = a;
   }

   public int getNumArrow()
   {
    return numArrow;
   }

   public void setNumGold(int a)
   {
    numGold = a;
   }

   public void setHint(String s)
   {
    hint = s;
   }

   public void setImage(int[][][] arr, int height, int width)
   {
    imageHeight = height;
    imageWidth = width;
    imagePixels = new int[height][width][3];

    for(int k = 0; k<3; ++k)
     for(int i =0; i< height; ++i)
      {
       for(int j = 0; j<width; ++j)
       {
        imagePixels[i][j][k] = arr[i][j][k];
       }
      }
   }

   public void setAverageScore(double a)
   {
    averageScore = a;
   }

   public void setStdDeviation(double a)
   {
    stdDeviation = a;
   }

   public int getImageWidth()
   {
    return imageWidth;
   }

   public int getImageHeight()
   {
    return imageHeight;
   }

   public boolean getFacingPit()
   {
       boolean retval = false;

       if(imagePixels != null)
       {
           // assuming (x,y) and (1,1) is the upper-left corner...
           // Pits are the only pictures with 255 0 0 at (17, 2)
           int pixel1[] = imagePixels[1   ][16   ];
           int pixel2[] = imagePixels[1   ][16+32];
           int pixel3[] = imagePixels[1+32][16   ];
           int pixel4[] = imagePixels[1+32][16+32];
       
           retval = ((pixel1[0] == 255 && pixel1[1] == 0 && pixel1[2] == 0) ||
                     (pixel2[0] == 255 && pixel2[1] == 0 && pixel2[2] == 0) ||
                     (pixel3[0] == 255 && pixel3[1] == 0 && pixel3[2] == 0) ||
                     (pixel4[0] == 255 && pixel4[1] == 0 && pixel4[2] == 0)   );
       }

       return retval;
   }

   public boolean getFacingWumpus()
   {
       boolean retval = false;

       if(imagePixels != null)
       {
           // assuming (x,y) and (1,1) is the upper-left corner...
           // Wumpii are the only pictures with 255 176 176 at (26, 8)
           int pixel1[] = imagePixels[7   ][25   ];
           int pixel2[] = imagePixels[7   ][25+32];
           int pixel3[] = imagePixels[7+32][25   ];
           int pixel4[] = imagePixels[7+32][25+32];
           retval = ((pixel1[0] == 255 && pixel1[1] == 176 && pixel1[2] == 176) ||
                     (pixel2[0] == 255 && pixel2[1] == 176 && pixel2[2] == 176) ||
                     (pixel3[0] == 255 && pixel3[1] == 176 && pixel3[2] == 176) ||
                     (pixel4[0] == 255 && pixel4[1] == 176 && pixel4[2] == 176)   );
       }

       return retval;
   }

   public boolean getFacingGold()
   {
       boolean retval = false;

       if(imagePixels != null)
       {
           // assuming (x,y) and (1,1) is the upper-left corner...
           // Pots of gold are the only pictures with 255 255 0 at (19,3)
           int pixel1[] = imagePixels[2   ][18   ];
           int pixel2[] = imagePixels[2   ][18+32];
           int pixel3[] = imagePixels[2+32][18   ];
           int pixel4[] = imagePixels[2+32][18+32];
           retval = ((pixel1[0] == 255 && pixel1[1] == 255 && pixel1[2] == 0) ||
                     (pixel2[0] == 255 && pixel2[1] == 255 && pixel2[2] == 0) ||
                     (pixel3[0] == 255 && pixel3[1] == 255 && pixel3[2] == 0) ||
                     (pixel4[0] == 255 && pixel4[1] == 255 && pixel4[2] == 0)   );
       }

       return retval;
   }

   public boolean getFacingObstruction()
   {
       boolean retval = false;

       if(imagePixels != null)
       {
           // assuming (x,y) and (1,1) is the upper-left corner...
           // Obstructions are the only pictures with 153 50 51 at (1,1)
           int pixel1[] = imagePixels[0   ][0   ];
           int pixel2[] = imagePixels[0   ][0+32];
           int pixel3[] = imagePixels[0+32][0   ];
           int pixel4[] = imagePixels[0+32][0+32];
           retval = ((pixel1[0] == 153 && pixel1[1] == 50 && pixel1[2] == 51) ||
                     (pixel2[0] == 153 && pixel2[1] == 50 && pixel2[2] == 51) ||
                     (pixel3[0] == 153 && pixel3[1] == 50 && pixel3[2] == 51) ||
                     (pixel4[0] == 153 && pixel4[1] == 50 && pixel4[2] == 51)   );
       }

       return retval;
   }

   public boolean getFacingBadcave()
   {
       return getFacingObstruction() ||
              getFacingPit();
   }

   public boolean getFacingImpasse()
   {
       return getFacingObstruction() ||
              getFacingWumpus() ||
              getFacingPit();
   }

   public int[][][] getImagePixels()
   {
    return imagePixels;
   }

   public int getNumGold()
   {
    return numGold;
   }

   public void setStatus(int a)
   {
    status = a;
   }

   public int getStatus()
   {
    return status;
   }

   public int getActionTaken()
   {
    return actionTaken;
   }

   public boolean getStench()
   {
    return stench;
   }

   public boolean getBreeze()
   {
    return breeze;
   }

   public boolean getGlitter()
   {
    return glitter;
   }

   public boolean getBump()
   {
    return bump;
   }

   public boolean getScream()
   {
    return scream;
   }

   public String getHint()
   {
    return hint;
   }

   public double getAverageScore()
   {
    return averageScore;
   }

   public double getStdDeviation()
   {
    return stdDeviation;
   }

}

