package wclient;

import java.util.Vector;
import wPercept.TransferPercept;
//import java.net.*;
import java.io.*;
import java.beans.*;

/**
Responsible for keeping any transferpercept objects that may
appear on the socket, so that they may be accessed by the
agent at any time.

MDS: Modified from PerceptKeeper to conform to Thermometer class in _Jess
in Action_ and to include bean interface.
*/


public class PerceptKeeperBean extends GenericBean implements Runnable
{
  private Communicator perceptcomm;
  private ObjectInputStream inpstream;
// The original PerceptKeeper kept a vector of percepts, to act as a
// buffer.  We merely retain the TransferPercept oject, and include
// accessors for the relevant TransferPercept properties so they'll
// appear as slots in our shadow facts.
  private TransferPercept percept;

  public PerceptKeeperBean(Communicator comm)
  {
    perceptcomm = comm;
    // We need a dummy percept so shadow facts will initialize without
    // the accessors throwing null pointer exceptions (since they all
    // work indirectly through the percept).
    percept = new TransferPercept();
    new Thread(this).start();
  }

  public boolean getStench() { return percept.getStench(); }

  public boolean getBreeze() { return percept.getBreeze(); }

  public boolean getGlitter() { return percept.getGlitter(); }

  public boolean getBump() { return percept.getBump(); }

  public boolean getScream() { return percept.getScream(); }

  public boolean getFacingWumpus() { return percept.getFacingWumpus(); }

  public boolean getFacingBadcave() { return percept.getFacingBadcave(); }

  public boolean getFacingImpasse() { return percept.getFacingImpasse(); }

  public String getHint() { return percept.getHint(); }

  public int getStatus() { return percept.getStatus(); }

  public int getScore() { return percept.getScore(); }

  public int getNumArrow() { return percept.getNumArrow(); }

  public int getNumGold() { return percept.getNumGold(); }

  public int getActionTaken() { return percept.getActionTaken(); }

    //  public void setActionTaken(int a) { percept.setActionTaken(a); }

    // The following is _not_ a bean property accessor. We don't want
    // the percept to appear as a JESS slot --- we only want the
    // shadow fact to update if one of the percept's _properties_ is
    // different than those of the old percept; we don't want rules
    // triggered just because we pulled a new percept from the
    // perceptcomm.
  public TransferPercept retrievePercept() { return percept; }

  public String makePropString(TransferPercept tp)
  {
      String s = new String(new Boolean(tp.getStench()).toString()) ;
      s = s + tp.getBreeze() + tp.getGlitter() + tp.getBump()
	  + tp.getScream() + tp.getFacingWumpus() + tp.getFacingBadcave()
      + tp.getFacingImpasse() + tp.getHint() + tp.getStatus() + tp.getScore()
	  + tp.getNumArrow() + tp.getNumGold() + tp.getActionTaken();
      //      System.err.println("Perceptstring is " + s);
      return s;
  }


    // We need to "manually" detect property changes and "synthesize"
    // a PropertyChangeEvent to force Jess to only update the shadow
    // fact once (it will update all slots if the property name is
    // null).
  public void fireChanges(TransferPercept oldPercept)
  {
      // We can't fire these changes individually, as it will cause
      // multiple single-slot shadow fact updates. So, instead, we
      // concatenate all the properties together in a single string,
      // detect that one has changed ourselves, and then fire a single
      // property change (no property named).
      String allOldProps = makePropString(oldPercept);
      String allNewProps = makePropString(percept);
      if (allOldProps.compareTo(allNewProps) != 0)
	  pcs.firePropertyChange(new PropertyChangeEvent(this, null,
							 allOldProps,
							 allNewProps));
      /* 
      pcs.firePropertyChange("stench", 
			     oldPercept.getStench(), 
			     percept.getStench());
      pcs.firePropertyChange("breeze", 
			     oldPercept.getBreeze(),
			     percept.getBreeze());
      pcs.firePropertyChange("glitter", 
			     oldPercept.getGlitter(),
			     percept.getGlitter());
      pcs.firePropertyChange("bump", 
			     oldPercept.getBump(),
			     percept.getBump());
      pcs.firePropertyChange("scream", 
			     oldPercept.getScream(),
			     percept.getScream());
      pcs.firePropertyChange("hint", oldPercept.getHint(), percept.getHint());
      pcs.firePropertyChange("status", 
			     oldPercept.getStatus(),
			     percept.getStatus());
      pcs.firePropertyChange("score", 
			     oldPercept.getScore(),
			     percept.getScore());
      pcs.firePropertyChange("numArrow", 
			     oldPercept.getNumArrow(),
			     percept.getNumArrow());
      pcs.firePropertyChange("numGold", 
			     oldPercept.getNumGold(),
			     percept.getNumGold());
      pcs.firePropertyChange("actionTaken", 
			     oldPercept.getActionTaken(),
			     percept.getActionTaken());

      */
  }

  public void run()
  {
  while(true)
    {
      TransferPercept oldPercept = percept;
      percept = (TransferPercept)perceptcomm.readInformation();
      // Lock this object
      fireChanges(oldPercept);
      // For some reason, checkAvailability() doesn't seem to work right.
      //      while (!perceptcomm.checkAvailability()) {
	  // Check for an available percept each couple seconds.
	 try{ 
	    Thread.sleep(2000); }
	  catch(InterruptedException ie) {
	     System.out.println(" Thread disturbed while sleeping");
	  }
	 //      }
    }
  }
}
