package wworld;
/**
   This class specializes the MovingObject class to include a State that
   each moving object can have.

   @author Kruti Mehta
*/

public class Actor extends MovingObject
{
   private int actorid;
   private String actorName;
   private ActorState state;

   /**
   Constructor that takes in the ObjectType, GridLocation in terms of x
   and y, the orientation o, the id number of the Actor, and the name
   assigned to the actor.

   @param objecttype, location of the object, actor ID, agent name
   */

   public Actor(int objType, int x, int y, int o, int idnumber, String name)
   {
    super(objType, x, y, o);
    actorid = idnumber;
    actorName = name;
    state = new ActorState(ActorState.ALIVE);
   }

   /**
   Returns the ID of the Actor as an integer.

   @return ID of the Actor
   */
   public int getActorId()
   {
    return actorid;
   }

   /**
   Returns the Name of the actor.

   @return Name of the actor.
   */
   public String getActorName()
   {
    return actorName;
   }

   /**
   Returns the State of the Actor as being ActorState.ALIVE,
   ActorState.DEAD or ActorState.VICTORIOUS.

   @return State of the agent (ALIVE, DEAD OR VICTORIOUS)
   */
   public ActorState getState()
   {
    return state;
   }

   /**
   Allows  the simulator to set the State of the Actor.

   @param state of the actor as integer.
   */
   public void setState(int s)
   {
    state.setActorState(s);
   }

}
