package wworld;

/**
   Defines the data and behaviour associated with an Agent. An agent is an
   alien to the Wumpus World, and is not part of the environment.

   @author Kruti Mehta
*/




public class Agent extends Actor
{
   private int maxArrowCount;
   private int arrowCount;
   private int goldCount;
   private int score;
   private int scoreForAction = -1;
   private int scoreForGold = 1000;
   private int scoreForDying = -1000;
   private boolean agentdonelasttime = false;

   /**
   Constructor. The agent can be allowed to have more number of arrows initially
   by changing the maxarr parameter in this constructor

   */
   public Agent(int objType, int x, int y, int o, int idnumber, String name, int maxArr)
   {
    super(objType, x, y, o, idnumber, name);
    maxArrowCount = maxArr;
    arrowCount = maxArr;
    goldCount = 0;
   }

   public Agent(int objType, GridPoint gp, int o, int idnumber, String name, int maxArr)
   {
    super(objType, gp.getPtX(), gp.getPtY(), o, idnumber, name);
    maxArrowCount = maxArr;
    arrowCount = maxArr;
    goldCount = 0;
   }

   /**
   Used to reset the agent parameters at the end of every trial.
   */

   public void reset()
   {
    arrowCount = maxArrowCount;
    goldCount = 0;
    setState(ActorState.ALIVE);
    score = 0;
    setOrientation(getOrigOrientation());
    setLocation(0, 0);
    agentdonelasttime = false;
   }

   /**

   */

   public int getArrowCount()
   {
    return arrowCount;
   }

   public int getGoldCount()
   {
    return goldCount;
   }

   public void incrementGoldCount()
   {
    goldCount = goldCount + 1;
   }

   public void decrementArrowCount()
   {
    arrowCount = arrowCount - 1;
   }

   public void addScoreForAction()
   {
    score = score + scoreForAction;
   }

   public void addScoreForGold()
   {
    score = score + scoreForGold;
   }

   public void addScoreForDying()
   {
    score = score + scoreForDying;
   }

   /**
   Acts out the actions taken by the agent, and also takes care of the
   agents' score.
   */


   public void act(int action)
   {
    if(action == Action.GOFORWARD)
      {
        move();
        addScoreForAction();
      }
    else if(action == Action.TURN_LEFT)
      {
        turnLeft();
        addScoreForAction();
      }
    else if(action == Action.TURN_RIGHT)
      {
        turnRight();
        addScoreForAction();
      }
    else if(action == Action.GRAB)
      {
        incrementGoldCount();
        addScoreForGold();
      }
    else if(action == Action.CLIMB)
      {
        setState(ActorState.VICTORIOUS);
        addScoreForAction();
        setAgentDoneLastTime(true);
      }
    else if(action == Action.SHOOT)
      {
        arrowCount = arrowCount - 1;
        addScoreForAction();
      }
    else if((action == Action.NO_OP) || (action == Action.NO_ACTION))
      {
        addScoreForAction();
      }
    else if(action == Action.SIT)
      {
        addScoreForAction();
      }
    return;
   }

   /**
   Handles the agent status and statistics when he dies.
   */

   public void die(GridPoint gp)
   {
    setLocation(gp);
    setState(ActorState.DEAD);
    setAgentDoneLastTime(true);
    addScoreForDying();
   }

   public int getScore()
   {
    return score;
   }

   public boolean isAgentDoneLastTime()
   {
    boolean agentdone= agentdonelasttime;

    if(agentdonelasttime)
    {
      setAgentDoneLastTime( false);
    }
    return agentdone;
   }

   public void setAgentDoneLastTime(boolean yesorno)
   {
    agentdonelasttime = yesorno;
   }
}
