package wworld;

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.io.*;
import java.util.Random;

/**
   This class contains the information about the objects and agents
   that are contained within the wumpus world grid. It provides methods
   to retrieve and manipulate the information contained within it.

   @author Kruti Mehta
*/

public class Environment
{
   /**
   Represents the size of the world described as size x size.
   */
   private int size;

   /**
   Represents the environment consisting of gold pieces, wumpii, pits
   and obstacles in a data structure representing a grid.
   */
   Vector[][] gridImage;

   /**
   A vector that carries a reference to all the wumpus in the world.
   */
   private Vector allWumpi;
   private Random randGen;

   /**
   Constructor
   */
   public Environment()
   {
    allWumpi = new Vector(3);
    randGen = new Random();
   }

   /**
   Returns the dimension of the world.
   @return worldsize.
   */
   public int getSize()
   {
   return size;
   }

   /**
   Returns the gridImage representing the Wumpus World environment.
   @return wumpus world.
   */
   public  Vector[][] getGridImage()
   {
     return gridImage;
   }

   /**
   Prints out the summary of the environment on standard output.
   */
   public void printEnvironment()
   {
    int i,j;
    char [][] displayWorld = new char[size*7 + 1][size*7+1];

    // Draw the outlines and borders;
    for(i=0;i<=size;++i)
     for(j=0;j<=size*7;++j)
      displayWorld[7*i][j] = '|';

    for(i=0;i<=size*7;++i)
     for(j=0;j<=size;++j)
      displayWorld[i][7*j] = '-';

    int maxSize = size * 7;
    // Draw the objects within the world
    for( i = 0; i < size; ++i)
      for( j = 0; j < size; ++j)
      {
       Vector v = new Vector();
       v = getObjectAtLocation(i, j);

       Enumeration e;

       if(!v.isEmpty())
       {
        e = v.elements();

        //System.out.print("(" + i + ", " + j + ") :");

        while(e.hasMoreElements())
        {
         Object obj = new Object();
         obj = e.nextElement();
         if(obj instanceof Wumpus)
         {
          Wumpus w = (Wumpus) obj;
          displayWorld[i*7 + 2][maxSize - (j*7)-2] = 'W';
          if(w.getOrientation() == Directions.DOWN)
           displayWorld[i*7 + 2][maxSize - (j*7)-2-1] = 'v';
          if(w.getOrientation() == Directions.LEFT)
           displayWorld[i*7 + 2-1][maxSize - (j*7)-2] = '<';
          if(w.getOrientation() == Directions.RIGHT)
           displayWorld[i*7 + 2+1][maxSize - (j*7)-2] = '>';
          if(w.getOrientation() == Directions.UP)
           displayWorld[i*7 + 2][maxSize - (j*7)-2+1] = '^';
         }
         else if(obj instanceof Pit)
         {
          displayWorld[i*7 + 3][maxSize - (j*7)-3] = 'P';
         }
         else if(obj instanceof Gold)
         {
          displayWorld[i*7 + 4][maxSize - (j*7)-4] = 'G';
         }
         else if(obj instanceof Obstacle)
         {
          for(int k=i*7+1;k<=i*7+6;++k)
           for(int l=j*7+1;l<=j*7+6;++l)
            displayWorld[k][maxSize - l] = '*';
         }
         else if(obj instanceof Agent)
         {
          Agent a = (Agent) obj;
          displayWorld[i*7+5][maxSize - (j*7)-5] = 'A';
          if(a.getOrientation() == Directions.DOWN)
           displayWorld[i*7 + 5][maxSize - (j*7)-5-1] = 'v';
          if(a.getOrientation() == Directions.LEFT)
           displayWorld[i*7 + 5-1][maxSize - (j*7)-5] = '<';
          if(a.getOrientation() == Directions.RIGHT)
           displayWorld[i*7 + 5+1][maxSize - (j*7)-5] = '>';
          if(a.getOrientation() == Directions.UP)
           displayWorld[i*7 + 5][maxSize - (j*7)-5+1] = '^';
         }



        } //end while
       }//endif
      }//end for
      System.out.println();
      System.out.println("_________________Updated World Display ________________");
      System.out.println();
      for(j=0;j<=size*7;++j)
      {
      for(i=0;i<=size*7;++i)
       System.out.print(displayWorld[i][j]);
      System.out.println();
     }
      System.out.println("_______________________________________________________");
   }//end function.


   /**
   Sets the size of the world.
   @param world size.
   */
   public void setSize(int s)
   {
    if(s >= 4 && s <=10)
       size = s;
    else
       size = 4;   //default size = 4
   }

   /**
   Determines whether a given gridpoint is within the bounds of the world.
   @param gridpoint to valid.
   @return TRUE if gridpoint within bounds, FALSE otherwise.
   */
   public boolean isValid(GridPoint gp)
   {
    boolean valid = true;
    int gx = gp.getPtX();
    int gy = gp.getPtY();
    if((gx >= size)|| (gx<0) || (gy<0) || (gy >= size) || isObstacleAt(gx, gy))
      {
        valid = false;
      }
    return valid;
   }

   /**
   Determines if a grid location is deadly for any agent entering it.
   A location is deadly if there is a wumpus or a pit at that location.
   @param Gridpoint for checking
   @return TRUE if wumpus or pit is at the gridpoint, FALSE otherwise.
   */
   public boolean isDeadly(GridPoint gp)
   {
    boolean deadly = false;
    if(isWumpusAt(gp.getPtX(),gp.getPtY()) || isPitAt(gp.getPtX(), gp.getPtY()))  //CHANGE ENV.ISWUMPUSAT TO TAKE GRIDPOINT AS PARAMETER.
      {
        deadly= true;
      }
    return deadly;
   }

   /**
   Returns the vector of objects at grid location (x, y).
   @param x and y parameters of the location
   @return Vector of objects at location (x,y).
   */
   public Vector getObjectAtLocation(int x, int y)
   {
    Vector v = new Vector();
    if(gridImage[x][y] instanceof Vector)
    {
     v = gridImage[x][y];
    }
    else
      System.out.println("EMPTY VECTOR BEING RETURNED");
    return v;
   }


   /**
   Reads in the environment from the input stream in and sets the
   gridimage according to the objects in the world
   @param Inputstream
   */
   public void readWorld(BufferedReader in) throws IOException
   {
    String temp;
    int t1, t2;

    String s = in.readLine();
    StringTokenizer t = new StringTokenizer(s, " \n\t\r");
    size = Integer.parseInt(t.nextToken());

    initGridImage();

    try {
	while(in.ready())
	    {
		s = in.readLine();
		t = new StringTokenizer(s, " \t\n\r");
		temp = t.nextToken();
		t1 = Integer.parseInt(t.nextToken());
		t2 = Integer.parseInt(t.nextToken());

		//add check here before insertion if(temp.instanceof object)...
		//gridImage[t1-1][t2-1].add(temp);

		if(temp.equalsIgnoreCase("w"))
		    {
			String t3 = new String(t.nextToken());
			Wumpus someWumpus = new Wumpus(GridObjectType.WUMPUS, t1, t2, Directions.RIGHT, 1, "wumpus", t3);
			gridImage[t1][t2].add(someWumpus);
			allWumpi.add(someWumpus);
			//gridImage[t1][t2].add(new Wumpus(GridObjectType.WUMPUS, t1, t2, Directions.RIGHT, 1, "wumpus", "random"));
		    }
		else if(temp.equalsIgnoreCase("p"))
		    {
			gridImage[t1][t2].add(new Pit(t1, t2));
		    }
		else if(temp.equalsIgnoreCase("g"))
		    {
			gridImage[t1][t2].add(new Gold(t1, t2));
		    }
		else if(temp.equalsIgnoreCase("o"))
		    {
			gridImage[t1][t2].add(new Obstacle(t1, t2));
		    }
	    } //end while
    } catch (NoSuchElementException e) {
	System.err.println("Error tokenizing line: " + s);
	e.printStackTrace();
    }
    setWumpusTargets();
   }//end readWorld()


   /**
   Sets targets for wumpii which have a mode that requires them to
   move to other locations with specific properties. For e.g. wumpus
   with mode GO_TO gold requires a target location with gold in it.
   */
   public void setWumpusTargets()
   {
    Vector goldLocations = new Vector();
    Vector pitLocations = new Vector();

    for(int i=0; i < size; ++i)
    {
     for(int j=0; j < size; ++j)
     {
      GridPoint gp = new GridPoint(i,j);
      if(isGoldAt(gp))
      {
       goldLocations.addElement(gp);
      }
      if(isPitAt(gp.getPtX(),gp.getPtY()))
       pitLocations.addElement(gp);
     }
    }

    int randnum;
    for(int i = 0; i < allWumpi.size(); ++i)
    {
     Wumpus wumpusAtI = (Wumpus) allWumpi.elementAt(i);
     if(wumpusAtI.getMode() == WumpusMode.GOTO_GOLD)
      {
       if(goldLocations.size() > 0)
       {
        randnum = randGen.nextInt(goldLocations.size());
        wumpusAtI.setTargetLoc((GridPoint) goldLocations.elementAt(randnum));
       }
       else
       {
        wumpusAtI.setTargetLoc(wumpusAtI.getLocation());
       }
      }
     else if(wumpusAtI.getMode() == WumpusMode.GOTO_PIT)
      {
       if(pitLocations.size() > 0)
       {
        randnum = randGen.nextInt(pitLocations.size());
        wumpusAtI.setTargetLoc((GridPoint) pitLocations.elementAt(randnum));
       }
       else
       {
        wumpusAtI.setTargetLoc(wumpusAtI.getLocation());
       }
      }
     }
   }

   /**
   Returns true if there is a wumpus at (x, y).
   @param location x and y
   @return TRUE if wumpus is at (x,y), FALSE otherwise.
   */
   public boolean isWumpusAt(int x, int y)
   {
    boolean wumpusExists = false;

    if(x>=0 && x<size && y>=0 && y<size)
    {
     Vector v = new Vector();
     v = getObjectAtLocation(x, y);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Wumpus)
        wumpusExists = true;
      }
     }//endif
    }
     return wumpusExists;
   }


   /**
   Returns the direction or facing of the wumpus if there is a wumpus at (x, y).
   Returns -99 otherwise.
   @param location x and y
   @return facing of the wumpus or -99 if no wumpus.
   */
   public int getWumpusFacing(GridPoint gp)
   {
    int wumpusFacing = -99;
    int x = gp.getPtX();
    int y = gp.getPtY();
    if(x>=0 && x<size && y>=0 && y<size)
    {
     Vector v = new Vector();
     v = getObjectAtLocation(x, y);


     for(int i = 0; i<v.size(); ++i)
     {
      if(v.elementAt(i) instanceof Wumpus)
      {
       Wumpus w = (Wumpus)v.elementAt(i);
       wumpusFacing = w.getOrientation();
      }
     }
    }
     return wumpusFacing;
   }


   /**
   Returns true if there is a wumpus in the 4 squares surrounding the
   grid point.
   @param Gridpoint
   @return TRUE if wumpus is in surrounding locations, FALSE otherwise.
   */
   public boolean isWumpusNear( GridPoint gp)
   {
    boolean wumpusNear = false;
    int wx = gp.getPtX();
    int wy = gp.getPtY();

    if(isWumpusAt(wx-1, wy) ||(isWumpusAt(wx+1, wy))||(isWumpusAt(wx,wy-1)) || (isWumpusAt(wx,wy+1)))
      wumpusNear = true;
    return wumpusNear;
   }

   /**
   Returns an integer indicating the position of a wumpus at a
   certain gridpoint, if at all the location contains a wumpus.
   */
   public int getWumpusAt(GridPoint gp)
   {
    int wump = -1;

    if(!isValid(gp))
    {
      return wump;
    }

    Vector v = getObjectAtLocation(gp.getPtX(), gp.getPtY());
    int i;
    for(i = 0; i<v.size(); ++i)
    {
      if(v.elementAt(i) instanceof Wumpus)
        {
          wump = i;
          break;
        }
    }

    return wump;
   }


   /**
   Positions the wumpii to their next calculated location.
   Called at the end of each step in the simulator.
   */
   public void processWumpi()
   {
    allWumpi.trimToSize();
    int s = allWumpi.size();

    for(int i = 0; i < s ; ++i)
    {
     Wumpus wumpusAtI = (Wumpus)allWumpi.elementAt(i);
     System.out.print("Wumpus at " + wumpusAtI.getLocation().toString() + "Action: ");
     removeWumpus(wumpusAtI);
     wumpusAtI.findAction(this);
     GridPoint wumpLoc = wumpusAtI.findNewLoc();
     wumpusAtI.act(isValid(wumpLoc));
     addWumpus(wumpusAtI);
    }
   }

   /**
   Processes the shooting of a wumpus from a  location pt  by an agent.
   @param Agent location and orientation
   @return TRUE if wumpus is shot, FALSE otherwise.
   */
   public boolean shootAtWumpus(GridPoint pt, int orientation)
   {
    int w;
    boolean flag = false;
    GridPoint gp = new GridPoint(pt.getPtX(), pt.getPtY());

    if(orientation == Directions.RIGHT)
    {
      while((gp.getPtX() < (size - 1))&&(gp.getPtX() >= 0) && !flag )  //dont have to check at gp, since if wumpus is at gp, agent is Dead.
      {
        gp.advanceRight();
        w = getWumpusAt(gp);
        System.out.println("IN ShootAtWumpus: The value of W is " + w);
        flag = removeWumpus(gp, w);
      }
    }
    else if(orientation == Directions.LEFT)
    {
      while(gp.getPtX() <= (size - 1)&&(gp.getPtX() > 0) && !flag )  //dont have to check at gp, since if wumpus is at gp, agent is Dead.
      {
        gp.advanceLeft();
        w = getWumpusAt(gp);
        flag = removeWumpus(gp, w);
      }
    }
    else if(orientation == Directions.DOWN)
    {
      while((gp.getPtY() <= (size - 1))&&(gp.getPtY() > 0) && !flag )  //dont have to check at gp, since if wumpus is at gp, agent is Dead.
      {
        gp.advanceDown();
        w = getWumpusAt(gp);
        System.out.println("IN ShootAtWumpus: The value of W is " + w);
        flag = removeWumpus(gp, w);
      }
    }
    else if(orientation == Directions.UP)
    {
      while((gp.getPtY() < (size - 1))&&(gp.getPtY() >= 0) && !flag )  //dont have to check at gp, since if wumpus is at gp, agent is Dead.
      {
        gp.advanceUp();
        w = getWumpusAt(gp);
        flag = removeWumpus(gp, w);
      }
    }

    return flag;
   }//end function shootatwumpus

   /**
   Removes wumpus from the given location.
   @param location and position
   @return TRUE if successful, FALSE otherwise.
   */
   private boolean  removeWumpus(GridPoint gp, int position)
   {
     boolean flag = false;

     if(position >= 0)
     {
      Vector v = getObjectAtLocation(gp.getPtX(), gp.getPtY());

      if(v.elementAt(position) instanceof Wumpus)
      {
        allWumpi.remove(v.elementAt(position));
        v.removeElementAt(position);
      }
      flag = true;
     }

     return flag;
   }

   /**
   Removes a specific wumpus object.
   */
   private void removeWumpus(Wumpus wObj)
   {
     Vector v = getObjectAtLocation(wObj.getLocation().getPtX(), wObj.getLocation().getPtY());

     v.remove(wObj);
   }

   /**
   Adds a specific wumpus object
   */
   private void addWumpus(Wumpus wObj)
   {
     Vector v = getObjectAtLocation(wObj.getLocation().getPtX(), wObj.getLocation().getPtY());

     v.addElement(wObj);
   }


   /**
   Returns true if there is a pit at location (x, y).
   @param location x and y
   @return TRUE if pit is found at (x,y), FALSE otherwise.
   */
   public boolean isPitAt(int x, int y)
   {
    boolean pitExists = false;

    if(x>=0 && x<size && y>=0 && y<size)
    {
     Vector v = new Vector();
     v = getObjectAtLocation(x, y);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Pit)
        pitExists = true;
      }
     }//endif
    }
     return pitExists;
   }

   /**
   Returns true if there is a pit in the 4 squares surrounding the
   grid point.
   @param Location gridpoint
   @return TRUE if pit is in surrounding gridpoints, FALSE otherwise.
   */
   public boolean isPitNear(GridPoint gp)
   {
    boolean pitNear = false;
    int wx = gp.getPtX();
    int wy = gp.getPtY();

    if(isPitAt(wx-1, wy) ||(isPitAt(wx+1, wy))||(isPitAt(wx,wy-1)) || (isPitAt(wx,wy+1)))
      {
       pitNear = true;
      }
    return pitNear;
   }

   /**
   Returns true if there is gold at the grid location gp.
   @param Gridpoint
   @return TRUE if the gold is present at the given location, FALSE otherwise.
   */
   public boolean isGoldAt(GridPoint gp)
   {

    boolean goldExists = false;
    int gx = gp.getPtX();
    int gy = gp.getPtY();
   //stem.out.println("First STatement in ISGOLDAT");
    if(gx>=0 && gx <size && gy>=0 && gy<size)
    {
     Vector v = new Vector();
     v = getObjectAtLocation(gx , gy);
     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Gold)
        {
          goldExists = true;
        }
      }
     }//endif
    }
     return goldExists;
   }

   /**
   Returns true if there is gold at the location (x,y)
   @param Location x and y coordinates.
   @return TRUE if the gold is present at the given location, FALSE otherwise.
   */
   public boolean isGoldAt(int x, int y)
   {
    boolean GoldExists = false;

    if(x>=0 && x<size && y>=0 && y<size)
    {
     Vector v = new Vector();
     v = getObjectAtLocation(x, y);

     for(int i = 0; i< v.size(); ++i)
     {
      if(v.elementAt(i) instanceof Gold)
        GoldExists = true;
     }

    }
    return GoldExists;
   }

   /**
   Returns true if there is a obstacle at location (x, y).
   @param location x and y coordinates
   @return TRUE if the obstacle is at (x,y), FALSE otherwise.
   */
   public boolean isObstacleAt(int x, int y)
   {
    boolean ObstacleExists = false;

    if(x>=0 && x<size && y>=0 && y<size)
    {
     Vector v = new Vector();
     v = getObjectAtLocation(x, y);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Obstacle)
        ObstacleExists = true;
      }
     }//endif
    }
    return ObstacleExists;
   }

   /**
   Removes gold pieces at location gp.
   @param Gridpoint
   */
   public void removeGoldAt(GridPoint gp)
   {
    int gx = gp.getPtX();
    int gy = gp.getPtY();

    if(gx>=0 && gx <size && gy>=0 && gy<size)
    {
     Vector v = new Vector();
     v = getObjectAtLocation(gx , gy);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();
      int i = 0;
      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Gold)
       {
        v.removeElementAt(i);
       }
       ++i;
      }
     }
    }
   }

   /**
   Populates the environment after reading it from file.
   @param filename containing world information
   */
   public void readWorldFromFile(String fileName)
   {
    try
      {
        BufferedReader in = new BufferedReader(new FileReader(fileName));
        for(int i = 0; i<size; ++i)
         for(int j=0; j<size; ++j)
          gridImage[i][j].removeAllElements();
        allWumpi.removeAllElements();
        readWorld(in);
        in.close();
        printEnvironment();
      }//end try
    catch(IOException e)
      {
        System.out.println(" Error : Could not open world file");
        System.exit(1);
      }
   }

   /**
   Initializes the GridImage.
   */
   private void initGridImage()
   {
    int i, j;
    gridImage = new Vector[size][size];
    for (i = 0; i < size; ++i)
      for(j = 0; j < size; ++j)
      {
       gridImage[i][j] = new Vector();
      }
   }

}
