package wworld;
import java.io.*;
import java.net.*;
import java.util.Vector;
import wPercept.TransferPercept;
import wAction.*;
//import PPMViewer.*;
/**
   The gameserver object controls the overall simulation of the wumpus world.
   It conducts the simulation, performs the action, updates the environment,
   etc.
   @author Kruti Mehta
*/
public class GameServer
{
  /** number of Trials specified for the game run. */
  private int numTrials;
  /** Current Trial number */
  private int currentTrial;
  private Environment env;
  /** Flag that is set to TRUE if the trial is over, FALSE otherwise */
  private boolean gameOver;
  /** Number of agents specified for this specific game run */
  private int numAgents;
  /** Vector of agent objects */
  private Vector allAgents;
  /** Vector of score objects */
  private Vector allScorer;
  /**Time allowed for clients to think. */
  private int sleepTime;
  private projectFrame wumpusFrame;
  /** Current number of steps */
  private int numSteps;
  /** Maximum number of steps allowed for each trial */
  public static int MAXSTEPS = 1000;

  public GameServer(Environment wumpusEnv, int trials, int currTrial, Vector vecScore, int waitTime,projectFrame pFrame)
  {
   env = new Environment();
   env = wumpusEnv;
   gameOver = false;
   allAgents = new Vector();
   numTrials = trials;
   currentTrial = currTrial;
   //allScorer = new Vector();
   allScorer = vecScore;
   sleepTime = waitTime;
   wumpusFrame = pFrame;
   numSteps = 0;
  }

  /**
  Used to reset attributes at the end of a game.
  @param wumpus environment and current trial number
  */
  public void reset(Environment wumpusEnv,  int currTrial)
  {
    env = wumpusEnv;
    gameOver = false;
    currentTrial = currTrial;
    numSteps = 0;
    allAgents.removeAllElements();
  }

  /**
  Conducts the simulation from beginning to the end of current trial.
  @param vector of clients, input and output streams
  */
  public void playGame1(Vector client, Vector cin, Vector cout,projectFrame myFrame, int currentTrial, int numTrials)
  {
   GridPoint initGp = new GridPoint(0, 0);
   System.out.println("_______________________________________________________");
   Vector allPercepts = new Vector();
   numAgents = client.size();
   int[] lastAction = new int[numAgents];

   //***********************************************************************
   //for showing the world in graphics form.
   wumpusFrame.displayWorld(env.getGridImage(), allAgents); //initial grid.
   //***********************************************************************

   for(int i = 0; i<numAgents ; ++i)
   {
    allAgents.addElement(new Agent(GridObjectType.AGENT, initGp, Directions.RIGHT, i, "sam", 1));
    allPercepts.addElement(new Percept(i));
   }
   gameOver = false;

   myFrame.setMaxSteps(MAXSTEPS);

   // Repeat until the game is over

   while(!gameOver && myFrame.getGameEnd() == false)
   {
    // Repeat the play game for each agent.
    for(int i = 0; i< numAgents ; ++i)
    {
    System.gc();
    // Populate the transfer percept for each client.
    Agent agent = (Agent)allAgents.elementAt(i);
    Percept percept = (Percept)allPercepts.elementAt(i);

    if((agent.getState().getActorState() == ActorState.ALIVE)||((agent.getState().getActorState() != ActorState.ALIVE) && agent.isAgentDoneLastTime()))
    {
     /* Get the percept..*/
     TransferPercept tp = new TransferPercept();

     GridPoint agentlocation = new GridPoint(agent.getLocation().getPtX(), agent.getLocation().getPtY());
     int agentorientation = agent.getOrientation();
     percept.perceive(env, agentlocation, agentorientation, allAgents, tp);

     tp.setScore(agent.getScore());
     tp.setNumArrow(agent.getArrowCount());
     tp.setNumGold(agent.getGoldCount());
     tp.setStatus(agent.getState().getActorState());
     tp.setPercept(percept.isStench(), percept.isBreeze(), percept.isGlitter(), percept.isBump(), percept.isScream());
     tp.setActionTaken(lastAction[i]);
     tp.setHint(percept.getHint());
     tp.setImage(percept.getImagePixels(), percept.getImageHeight(),percept.getImageWidth());

     sendToStream((Socket)client.elementAt(i),
                  (ObjectOutputStream)cout.elementAt(i), tp);
    } // end of if statement
   } //end for statement

   // After writing the percepts, sleep for sleepTime before getting client actions
   try
   {
   Thread.sleep(sleepTime);
   }
   catch(InterruptedException ie)
   {
    System.out.println("GameServer Error: Thread disturbed while sleeping.");
   }

   // Get the actions from each client that is alive.
   Vector allActions = new Vector();

   for(int i = 0; i< numAgents ; ++i)
   {
    Agent agent = (Agent)allAgents.elementAt(i);
    int motion;

    if((agent.getState().getActorState() == ActorState.ALIVE)||((agent.getState().getActorState() != ActorState.ALIVE) && agent.isAgentDoneLastTime()))
    {
      Percept percept = (Percept)allPercepts.elementAt(i);
      motion = getAction(agent, (ObjectInputStream)cin.elementAt(i));
    }
    else
    {
      motion = Action.NO_OP;
    }
    Integer someInt = new Integer(motion);
    allActions.add(someInt);
    lastAction[i] = motion;
    Action maction = new Action();
   }

   //System.out.println("_____________________________________________________");
   System.out.println("Trial Number: " + currentTrial);
   System.out.println("Step Number: " + numSteps);

   // Act the action that has been received from the client.
   for(int i = 0; i< numAgents ; ++i)
   {
    Agent agent = (Agent)allAgents.elementAt(i);
    if(agent.getState().getActorState() == ActorState.ALIVE)
    {
      Percept percept = (Percept)allPercepts.elementAt(i);
      Integer actionInt= (Integer)allActions.elementAt(i);
      int action = actionInt.intValue();
      Action maction = new Action();
      System.out.println("Agent " + i + " Action: " + maction.toString(action));
      doAction(agent, percept, action);
    }
   }
   //System.out.println("______________________________________________________");
   env.processWumpi();
   //env.printEnvironment();
   wumpusFrame.displayWorld(env.getGridImage(), allAgents);
   ++numSteps;
   myFrame.setCurrentStep(numSteps);

   // Test to see if all the clients are not alive and the trial has ended.
   int count = 0;
   for(int j = 0; j<numAgents; ++j)
   {
    Agent agentj = (Agent)allAgents.elementAt(j);
    if(agentj.getState().getActorState()!= ActorState.ALIVE)
     {
      count++;
      lastAction[j] = Action.END_TRIAL;
     }
    }

    if(count == numAgents || numSteps == MAXSTEPS)
     gameOver = true;
   }// End of while loop

   // The trial has ended. Provide the appropriate percepts to the client.
   for(int i = 0; i< numAgents ; ++i)
   {
    // Populate the transfer percept for each client after the end of trial.
    Agent agent = (Agent)allAgents.elementAt(i);
    Percept percept = (Percept)allPercepts.elementAt(i);

    TransferPercept tp = new TransferPercept();
    GridPoint agentlocation = new GridPoint(agent.getLocation().getPtX(), agent.getLocation().getPtY());
    int agentorientation = agent.getOrientation();
    percept.perceive(env, agentlocation, agentorientation, allAgents, tp);

    tp.setScore(agent.getScore());
    tp.setNumArrow(agent.getArrowCount());
    tp.setNumGold(agent.getGoldCount());
    tp.setStatus(agent.getState().getActorState());
    tp.setPercept(percept.isStench(), percept.isBreeze(), percept.isGlitter(), percept.isBump(), percept.isScream());
    if(currentTrial != numTrials)
     tp.setActionTaken(Action.END_TRIAL);
    else
     tp.setActionTaken(Action.END_SIMULATION);
    tp.setHint(percept.getHint());
    tp.setImage(percept.getImagePixels(), percept.getImageHeight(),percept.getImageWidth());

    // Calculate the average score and standard deviation and send it to tp.
    // This should only happen if this is the end of the simulation

    ScoreKeeper agentiScorer = (ScoreKeeper)allScorer.elementAt(i);
    agentiScorer.gameScore[currentTrial] = agent.getScore();
    if(currentTrial == numTrials)
    {
     agentiScorer.calculateAvgScore();
     agentiScorer.calculateStdDeviation();
     tp.setAverageScore(agentiScorer.getAverageScore());
     tp.setStdDeviation(agentiScorer.getStdDeviation());
     System.out.println("------------- GAME END STATISTICS ----------------");
     System.out.println("------------- Client " + i + " Statistic ---------");
     System.out.println("Average Score: " + agentiScorer.getAverageScore());
     System.out.println("Standard Deviation: " + agentiScorer.getStdDeviation());
     System.out.println("--------------------------------------------------");
    }
    agent.reset();
    sendToStream((Socket)client.elementAt(i),
                  (ObjectOutputStream)cout.elementAt(i), tp);
   } //end for statement
}

  /**
  Prints all the statistics for each agent at the end of each trial, and
  at the end of all the trials.
  */
public void printStats()
{
  for(int j = 0; j< numAgents; ++j)
  {
     ScoreKeeper agentjScorer = (ScoreKeeper)allScorer.elementAt(j);
     agentjScorer.calculateAvgScore();
     agentjScorer.calculateStdDeviation();
     System.out.println(agentjScorer.averageScore);
     System.out.println(agentjScorer.stdDeviation);
  }
}

 /**
    Performs the action specified by the agent, and sets the necessary
    variable in the Percept.
    @param agent, percept for the agent and action specified by the agent
 */
public void doAction(Agent agent, Percept percept, int action)
{
  if(agent.getState().getActorState() == ActorState.ALIVE)
  {
    GridPoint agentLoc = agent.getLocation();

    if(action == Action.GOFORWARD)
    {
      GridPoint gp = new GridPoint(agentLoc.getPtX(), agentLoc.getPtY());
      switch(agent.getOrientation())
      {
        case Directions.LEFT:
          gp.advanceLeft();
          break;
        case Directions.RIGHT:
          gp.advanceRight();
          break;
        case Directions.DOWN:
          gp.advanceDown();
          break;
        case Directions.UP:
          gp.advanceUp();
          break;
      }//end switch
      System.out.println("GPX: " + gp.getPtX() + " GPY: " + gp.getPtY());
    /* Act the Action.. */
      if( env.isValid(gp) && !env.isDeadly(gp))
      {
        agent.act(action);
      }
      else if (env.isValid(gp) && env.isDeadly(gp))
      {
        agent.die(gp);
      }
      else if(!env.isValid(gp))
      {
        action = Action.NO_OP;
        agent.act(action);
        percept.setBumpedLastTime();
      }

      System.out.println("agentx: " + agent.getLocation().getPtX() + " agenty: " + agent.getLocation().getPtY());
  }//end if Goforward

  else if ((action == Action.TURN_LEFT) || (action == Action.TURN_RIGHT))
  {
    if(env.isDeadly(agent.getLocation()))
    {
     agent.die(agent.getLocation());
    }
    else
     agent.act(action);
  }

  else if (action == Action.GRAB)
  {
    if(env.isDeadly(agent.getLocation()))
    {
     agent.die(agent.getLocation());
    }
    else
    {
     if(env.isGoldAt(agentLoc))
     {
       agent.act(action);
       System.out.println("Agent " + agent.getActorId() + " has " + agent.getGoldCount() + "gold piece/s");
       env.removeGoldAt(agentLoc);
     }
     else
     {
       System.out.println("Agent " + agent.getActorId() + " tried to grab nonexistent gold");
       action = Action.NO_OP;
       agent.act(action);
     }
    }
  }

  else if (action == Action.CLIMB)
  {
    if((agentLoc.getPtX()== 0) && (agentLoc.getPtY() == 0))
    {
      System.out.println("Agent " + agent.getActorId() + " is climbing out of cave");
      agent.act(action);
    }
    else
    {
      System.out.println("Agent " + agent.getActorId() + " cannot climb out of this location.");
      action = Action.NO_OP;
      agent.act(action);
    }
  }
  else if (action == Action.SHOOT)
  {
    boolean didWumpusDie;
    System.out.println("Agent " + agent.getActorId() + " is shooting");
    if(env.isDeadly(agent.getLocation()))
    {
     agent.die(agent.getLocation());
    }
    else
     {
      if(agent.getArrowCount() >= 1)
      {
        agent.act(action);
        didWumpusDie = env.shootAtWumpus(agentLoc, agent.getOrientation());
        if(didWumpusDie)
        {
          System.out.println("Agent " + agent.getActorId() + " killed the wumpus!" + "didwumpdie = " + didWumpusDie);
          percept.setWumpusDiedLastTime();
        }
        else
          System.out.println("Agent " + agent.getActorId() + " missed the wumpus!"+ "didwumpdie = " + didWumpusDie);

      }
      else if(agent.getArrowCount() <=0)
      {
       System.out.println("Agent " + agent.getActorId() + " is out of arrows!");
       action = Action.NO_OP;
       agent.act(action);
      }
     }
  }

  else if (action == Action.SIT || action == Action.NO_ACTION)
  {
    if(env.isDeadly(agent.getLocation()))
    {
     agent.die(agent.getLocation());
    }
    else
     agent.act(action);
  }
}
}

  /**
  Does the necessary communication with the client and gets the
  action specified by the clients.
  @param socket with ith client and Inputstream
  @return action coming from the ith client
  */
  public int getAction(Agent agent, ObjectInputStream in)
  {
    int action = Action.NO_ACTION;
    try
    {
    try {
      for (int numTries=0; numTries<60000/sleepTime && in.available()<=0; numTries++)
        Thread.sleep(sleepTime);
    } catch(InterruptedException ie) {
      System.out.println("GameServer Error: Thread disturbed while sleeping.");
    }
    if (in.available()<=0) {  // Client didn't respond; kill its agent
      GridPoint agentLoc = agent.getLocation();
      GridPoint gp = new GridPoint(agentLoc.getPtX(), agentLoc.getPtY());
      agent.die(gp);
      return Action.NO_OP;
    }
      action = in.readInt();
    }
    catch(IOException ioe)
    {
      System.out.println("GAMESERVER: IOException in getAction.");
    }
    return action;
  }

  /**
  Sends the specified obj over the socket to the specified client.
  */
  public void sendToStream(Socket iclient, ObjectOutputStream out, Object obj)
  {
   try
   {
     //System.out.println("Writing transferpercept...");
     out.writeObject(obj);
     out.flush();
     //System.out.println("Successful");
   }
   catch (IOException ioe)
   {
    System.out.println("Exception in sendToStream.1.");
   }
  }

  /**
  Sends an integer over the socket connection to the client.
  */
  public void sendIntToStream(Socket iclient, ObjectInputStream in, ObjectOutputStream out, int i)
  {
   try
   {
     //System.out.println("Writing Int...");
     out.writeInt(i);
     out.flush();
     //System.out.println("Successful");
   }
   catch (IOException ioe)
   {
     System.out.println("Exception in sendToStream.1.");
   }
  }

  /** prints the prompt */
  public static void printPrompt(String prompt)
  {
   System.out.print(prompt + " ");
   System.out.flush();
  }


  /**
  Reads a line from the standard input
  */
  public static String readLine()
  {
   int ch;
   String r = "";
   boolean done = false;
   while (!done)
   {
     try
     {
       ch = System.in.read();
       if (ch < 0 || (char)ch == '\n')
         done = true;
       else if ((char)ch != '\r')
         r = r + (char) ch;
     }
     catch(java.io.IOException e)
     {
      done = true;
     }
   }
   return r;
  }

  public static String readLine(String prompt)
  {
   printPrompt(prompt);
   return readLine();
  }

}//end class
