package wworld;
/**
   Class specifying object representing each GridPoint in the world.
   @author Kruti Mehta
*/
public class GridPoint implements Cloneable
{
   /** Describes the horizontal position of the location in the grid. */
   private int x;
   /** Describes the vertical position of the location in the grid. */
   private int y;

   public GridPoint()
   {
    x=y=0;
   }

   public GridPoint(int newx, int newy)
   {
    x = newx;
    y = newy;
   }

   /**
   Returns an Object that is a copy of this grid point object.
   */
   public Object clone()
   {
    try{
        GridPoint copy = (GridPoint)super.clone();
        copy.x = x;
        copy.y = y;
        return copy;
       }
    catch (CloneNotSupportedException e)
    {
     throw new Error("GridPoint: Error in cloning point");
    }
   }

   /**
   Used to set the x position of this Object.
   */
   public void setPtX(int newx)
   {
    x =  newx;
   }

   /**
   Used to set the y position of this object.
   */
   public void setPtY(int newy)
   {
    y = newy;
   }

   /**
   Sets the attributes to the indicated new values.
   */
   public void setGridPoint(int newx, int newy)
   {
    x = newx;
    y = newy;
   }

   /**
   Sets the attributes of this GridPoint to be the same as those of
   the GridPoint passed to it.
   */
   public void setGridPoint(GridPoint p)
   {
    x = p.x;
    y = p.y;
   }

   /**
   Returns the horizontal position of the location represented by this
   GridPoint.
   */
   public int getPtX()
   {return x;}

   /**
   Returns the vertical position of the location represented by
   this GridPoint.
   */
   public int getPtY()
   {return y;}

   /**
   Sets the attributes of this gridpoint to point to the location to
   its left.
   */
   public void advanceLeft()
   {
    x = x-1;
   }

   /**
   Sets the attributes of this gridpoint to point to the location to
   its right.
   */
   public void advanceRight()
   {
    x = x+1;
   }

   /**
   Sets the attributes of this gridpoint to point to the location to
   the location above it.
   */
   public void advanceUp()
   {
    y = y+1;
   }

   /**
   Sets the attributes of this gridpoint to point to the location below it.
   */
   public void advanceDown()
   {
    y = y-1;
   }

   /**
   Represents this object as a String of the form (x, y).
   */
   public String toString()
   {
    String s;
    s = "(" + x + ", " + y + ")";
    return s;
   }
}