package wworld;

/**
   The class specifying object MovingObject. This object extends GridObject
   and additionally specifies the orientation of the object. This class
   is extended by Agent and Wumpus object classes.
   @author Kruti Mehta
*/
public class MovingObject extends GridObject
{
   /** Current direction or facing of the object */
   private Directions orientation;
   /** Direction or facing of the object when game started. */
   private Directions origOrientation;

   /**
   Constructor
   @param Type of the object, location parameters and Direction.
   */
   public MovingObject(int objType, int x, int y, int o)    //orientation o
   {
    super(objType, x, y);
    orientation = new Directions(o);
    origOrientation = new Directions(o);
   }

   /**
   Returns the current direction of the object as integer.
   */
   public int getOrientation()
   {
    return orientation.getFacing();
   }

   /**
   Returns the current direction of the object as Directions.
   */
   public Directions getOrientationDirection()
   {
    return orientation;
   }

   /**
   Sets the direction or facing of the object.
   */
   public void setOrientation(int o)
   {
    orientation.setFacing(o);
   }

   /**
   Returns the original direction or facing of the object.
   */
   public int getOrigOrientation()
   {
    return origOrientation.getFacing();
   }

   /**
   Performs the right turn and sets the new location and orientation of
   the object.
   */
   public void turnRight()     //0->3; 3->2; 2->1 ; 1->0
   {                           //4->3; 6->2; 5->1 ; 4->0
    int x = orientation.getFacing();
    if(x > 1)
      x = x-1;
    else
      x = 4;
    orientation.setFacing(x);
   }

   /**
   Performs the left turn and sets the new location and orientation of
   the object.
   */
   public void turnLeft()
   {
    int x = orientation.getFacing();
    if(x < 4)
      x = x+1;
    else
      x = 1;
    orientation.setFacing(x);
   }

   /**
   Performs the move action (going forward action). The result of the
   action is dependent on the current location and the current orientation
   of the object. Also, sets the new Location of the object after the move.
   */
   public void move()
   {
     switch(orientation.getFacing())
     {
      case(Directions.RIGHT):
         this.getLocation().advanceRight();
         break;
      case(Directions.LEFT):
         this.getLocation().advanceLeft();
         break;
      case(Directions.UP):
         this.getLocation().advanceUp();
         break;
      case(Directions.DOWN):
         this.getLocation().advanceDown();
         break;
      default:
         System.out.println("MovingObject: Logic Error in switch.");
         break;
      }
   }

}