package wworld;

import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
import java.net.*;
import java.util.Random;

/**
Class responsible for the simulation setup, start, end and command line
arguments.
Contains the Main() function.

@author Kruti Mehta
*/

public class WorldApplication
{
  int maxClients = 1;
  //int numTrials;

  ServerSocket ssock = null;
  Vector csock = new Vector();
  Vector cin = new Vector();
  Vector cout = new Vector();
  Vector vecScorers = new Vector();
  int connects = 0;
  projectFrame myFrame;

  public WorldApplication()
  {
    myFrame = new projectFrame();
    myFrame.show();
  }

  static Environment readEnv(BufferedReader in) throws IOException
  {
    Environment env = new Environment();
    env.readWorld(in);
    env.setWumpusTargets();
    return env;
  }

  public static void main(String[] args) throws java.io.IOException
  {
    WorldApplication wa = new WorldApplication();
    int numTrials = 1;
    int currentTrial = 1;
    int worldsize = 4;
    //int numClients;
    int seed = 1234;
    String worldFileName = "world.txt";
    int waitTime = 1000;
    int portNum = 10000;
    boolean createRandWorld = false;
    boolean givenSeed = false;


    for(int i = 0; i < args.length; ++i)
    {
     String str = args[i];

     if(str.equals("-t"))
       {
        numTrials = Integer.parseInt(args[i+1]);
        System.out.println("Number of Trials: " + numTrials);
        ++i;
       }
     else if(str.equals("-s"))
       {
        worldsize = Integer.parseInt(args[i+1]);
        System.out.println("World Size: " + worldsize);
        if(!(worldsize>=1 && worldsize<=10))
          {
           System.out.println("INVALID WORLD SIZE");
           System.exit(1);
          }
        createRandWorld = true;
        ++i;
       }
     else if(str.equals("-c"))
       {
        wa.maxClients = Integer.parseInt(args[i+1]);
        System.out.println("Number of clients: " + wa.maxClients);
        ++i;
       }
     else if(str.equals("-r"))
       {
        seed = Integer.parseInt(args[i+1]);
        System.out.println("Seed: " + seed);
        createRandWorld = true;
        givenSeed = true;
        ++i;
       }
     else if(str.equals("-f"))
       {
        worldFileName = args[i+1];
        /*File foo = new File("a");

        //String filepath = foo.getCanonicalPath();
        //String subpath = filepath.substring(0, filepath.length()-2);
        StringBuffer strbuf = new StringBuffer(filepath);
        strbuf = strbuf.deleteCharAt(strbuf.length()-1);
        System.out.println(strbuf.toString());
        String s = strbuf.toString();
        worldFileName = s.concat(worldFileName);
        System.out.println(worldFileName); */
        ++i;
       }
     else if(str.equals("-d"))
       {
       }
     else if(str.equals("-w"))
       {
        waitTime = Integer.parseInt(args[i+1]);
        System.out.println("Wait time: " + waitTime);
        ++i;
       }
     else if(str.equals("-a"))
       {
        portNum = Integer.parseInt(args[i+2]);
        i += 2;
       }
     else
       {
        System.out.println("Illegal option: " + str.toString());
        //System.exit(-1);
       }

    }

    Environment ke = new Environment();

    if(createRandWorld && givenSeed)
     {
      generateRandWorld(seed, worldsize);
      worldFileName = "random_world";
     }
    else if(createRandWorld && !givenSeed)
     {
      Random rand = new Random();
      seed = rand.nextInt();
      generateRandWorld(seed, worldsize);
      worldFileName = "random_world";
     }

    ke.readWorldFromFile(worldFileName);
    worldsize = ke.getSize();

    for(int i = 0; i < wa.maxClients; ++i)
    {
      wa.vecScorers.addElement(new ScoreKeeper(numTrials));
    }

    wa.myFrame.setWorldSize(worldsize);
    GameServer myGameServer = new GameServer(ke, numTrials, currentTrial, wa.vecScorers, waitTime, wa.myFrame);

    wa.makeSocket(portNum);
    while(wa.myFrame.getGameStart() == false);

    wa.myFrame.setNumTrials(numTrials);
    while(currentTrial <= numTrials && wa.myFrame.getGameEnd() == false)
    {
      wa.myFrame.setCurrentTrial(currentTrial);
      wa.myFrame.setCurrentStep(0);
      myGameServer.playGame1(wa.csock, wa.cin, wa.cout,wa.myFrame, currentTrial, numTrials);
      ke.readWorldFromFile(worldFileName);
      ++currentTrial;
      myGameServer.reset(ke, currentTrial);
    }

    wa.myFrame.dispose();

    wa.printStats();


  } //end main



public void printStats()
{
  for(int j = 0; j< maxClients; ++j)
  {
     ScoreKeeper agentjScorer = (ScoreKeeper)vecScorers.elementAt(j);
     agentjScorer.calculateAvgScore();
     agentjScorer.calculateStdDeviation();
     System.out.println(agentjScorer.averageScore);
     System.out.println(agentjScorer.stdDeviation);
  }
}

/**
Sets up communication mechanism on the server side.
*/

public void makeSocket(int portNumber)
{
 try
    {
     //create the server socket
     ssock = new ServerSocket(portNumber, maxClients);

     System.out.println("WumpusWorld server waiting for " + maxClients
			+ " connection(s)...");

     while (connects < maxClients)
     {
      //wait for connection
      csock.addElement(ssock.accept());
      System.err.println("Accepting connection from client " + connects);
      connects++;
     }

     for(int i=0;i<maxClients;++i)
     {
       Socket iclient = (Socket)csock.elementAt(i);
       ObjectOutputStream out = new ObjectOutputStream(iclient.getOutputStream());
       cout.addElement(out);
       ObjectInputStream in = new ObjectInputStream(iclient.getInputStream());
       cin.addElement(in);
     }

     System.out.println("All connections made; sending client IDs...");

     for(int i = 0; i< maxClients; ++i)
     {
      ObjectOutputStream outStr = (ObjectOutputStream)cout.elementAt(i);
      outStr.writeInt(i);
      }

     System.out.println("Client IDs sent; ready to start game.");

     //serverSocket.close();
    }
    catch(IOException ioe)
    {
     System.err.println("Error making sockets in WumpusWorld server.");
     ioe.printStackTrace();
     System.exit(1);
    }
}

/**
Reads in the wumpus world from the file.
*/

public void readWorldFromFile(Environment env, BufferedReader in)
{
    try
    {
      //BufferedReader in = new BufferedReader(new FileReader("\\JBuilder3\\myprojects\\world.txt"));
      env = readEnv(in);
      in.close();
      env.printEnvironment();
    }//end try
    catch(IOException e)
    {
      System.out.println(" Error : Could not open world file");
      System.exit(1);
    }
}

  /**
  Generates a random world when called.
  */

   public static void generateRandWorld(int seed, int size)
   {
    int i, x, y, mode;
    boolean occupied[][] = new boolean[size][size];

    try
    {
     PrintStream out = new PrintStream (new FileOutputStream("random_world"));
     out.println(size);
     Random randGen = new Random(seed);

     int wumpi = randGen.nextInt(size);
     int pits = randGen.nextInt(size);
     int golds = randGen.nextInt(size);
     int obstacles = randGen.nextInt(size);

     for( i = 0; i < wumpi ; ++i)
     {
       x = randGen.nextInt(size);
       y = randGen.nextInt(size);


       if(!(x ==1 && y == 1))
       {
        occupied[x][y] = true;
        mode = randGen.nextInt(6);
        ++mode;
        out.print("w " + x + " " + y + " ");
        out.println(WumpusMode.toString(mode));
       }
     }

     for( i = 0; i < pits ; ++i)
     {
       x = randGen.nextInt(size);
       y = randGen.nextInt(size);

       if(!(x ==1 && y == 1))
       {
        occupied[x][y] = true;
        out.println("p " + x + " " + y);
       }
     }

     for( i = 0; i < golds ; ++i)
     {
       x = randGen.nextInt(size);
       y = randGen.nextInt(size);
       occupied[x][y] = true;
       out.println("g " + x + " " + y);
     }

     for( i = 0; i < obstacles ; ++i)
     {
       x = randGen.nextInt(size);
       y = randGen.nextInt(size);
       if(!occupied[x][y])
       {
        out.println("o " + x + " " + y);
       }
     }

     out.close();
    }
    catch(FileNotFoundException fe)
    {
     System.out.println("GenerateRandomWorld: File not found" + fe.getMessage());

    }
    catch(IOException ioe)
    {
     System.out.println("GenerateRandomWorld:IOException while creating file.");
    }
  }


}   //end public class WorldApplication
