package wworld;
import wAction.Action;
import java.util.Random;

/**
Class Responsible for the behaviour of a Wumpus.
*/

public class Wumpus extends Actor
{


   private int mode;
   private Random randGen;
   private int action;
   private GridPoint origTarget;
   private GridPoint targetLoc;

 /**
 Constructor
 */
   public Wumpus(int objType, int x, int y,int o, int idnumber, String name, String itsmode)
   {
    super(objType, x, y, o, idnumber, name);
    setMode(itsmode);
    randGen = new Random();
   }


   /**
   Returns the mode of a wumpus.
   See Class WumpusMode for various defined modes.
   */

   public int getMode()
   {
    return mode;
   }


   public void setTargetLoc(GridPoint gp)
   {
    origTarget = new GridPoint(gp.getPtX(),gp.getPtY());
    targetLoc = new GridPoint(gp.getPtX(),gp.getPtY());

   }

   public  void findAction(Environment env)
   {
    if(mode == WumpusMode.SPIN)
      {findSpinAction();}
    else if(mode == WumpusMode.STAY)
      {findStayAction();}
    else if(mode == WumpusMode.GOTO_GOLD)
      {findGotoAction(env);}
    else if(mode == WumpusMode.GOTO_PIT)
      {findGotoAction(env);}
    else if(mode == WumpusMode.WALK)
      {findWalkAction(env);}
    else
      {findRandomAction(env);}

    System.out.println(Action.printAction(action));

   }

   public void findSpinAction()
   {
    action = Action.TURN_LEFT;
   }

   public void findStayAction()
   {
    action = Action.SIT;
   }

   public void findGotoAction(Environment env)
   {
    GridPoint wumpLoc = this.getLocation();
    GridPoint nextLoc = new GridPoint(wumpLoc.getPtX(),wumpLoc.getPtY());

    if(targetLoc.getPtX() == wumpLoc.getPtX() && targetLoc.getPtY() == wumpLoc.getPtY())
    {
     if(origTarget.getPtX() == targetLoc.getPtX() && origTarget.getPtY() == targetLoc.getPtY())
     {
      action = Action.SIT;
      return;
     }
     else
     {
      targetLoc.setPtX(origTarget.getPtX());
      targetLoc.setPtY(origTarget.getPtY());
     }
    }

    moveforward(nextLoc);

    if(env.isValid(nextLoc))
    {
     if(this.getOrientation() == Directions.RIGHT && targetLoc.getPtX() > wumpLoc.getPtX())
      action = Action.GOFORWARD;
     else if(this.getOrientation() == Directions.LEFT && targetLoc.getPtX() < wumpLoc.getPtX())
      action = Action.GOFORWARD;
     else if(this.getOrientation() == Directions.UP && targetLoc.getPtY() > wumpLoc.getPtY())
      action = Action.GOFORWARD;
     else if(this.getOrientation() == Directions.DOWN && targetLoc.getPtY() < wumpLoc.getPtY())
      action = Action.GOFORWARD;
     else
      action = Action.TURN_LEFT;
    }
    else
    {
     if(env.isObstacleAt(nextLoc.getPtX(),nextLoc.getPtY()))
      {
       int randnum;
       randnum = randGen.nextInt(env.getSize());
       targetLoc.setPtX(randnum);
       randnum = randGen.nextInt(env.getSize());
       targetLoc.setPtY(randnum);
      }
     action = Action.TURN_LEFT;
     }
   }

   public void findWalkAction(Environment env)
   {

    GridPoint wumpLoc = this.getLocation();
    GridPoint nextLoc = new GridPoint(wumpLoc.getPtX(),wumpLoc.getPtY());

    moveforward(nextLoc);

    if(env.isValid(nextLoc))
     action = Action.GOFORWARD;
    else
     action = Action.TURN_LEFT;
   }

   public void findRandomAction(Environment env)
   {

    int randnum = randGen.nextInt(4);
    GridPoint wumpLoc = this.getLocation();
    GridPoint nextLoc = new GridPoint(wumpLoc.getPtX(),wumpLoc.getPtY());

    moveforward(nextLoc);

    if(randnum == 0 && (env.isValid(nextLoc)))
     action = Action.GOFORWARD;
    else if(randnum == 1)
     action = Action.TURN_LEFT;
    else if(randnum == 2)
     action = Action.TURN_RIGHT;
    else
     action = Action.SIT;
   }


   public GridPoint findNewLoc()
   {
    GridPoint newWumpusLoc = (GridPoint)this.getLocation().clone();

    if(action == Action.GOFORWARD)
    {
     moveforward(newWumpusLoc);
    }

    return newWumpusLoc;
   }



   public void act(boolean actOrNot)
   {
    if(actOrNot)
    {
     if(action == Action.GOFORWARD)
       this.move();
     else if(action == Action.TURN_LEFT)
       this.turnLeft();
     else if(action == Action.TURN_RIGHT)
       this.turnRight();
    }
    return;
   }



   public void moveforward(GridPoint wumploc)
   {
    int x, y;

    x = wumploc.getPtX();
    y = wumploc.getPtY();

    if(this.getOrientation() == Directions.LEFT)
    {
      wumploc.advanceLeft();
    }
    else if(this.getOrientation() ==Directions.RIGHT)
    {
      wumploc.advanceRight();
    }
    else if(this.getOrientation() == Directions.UP)
    {
      wumploc.advanceUp();
    }
    else if(this.getOrientation() == Directions.DOWN)
    {
      wumploc.advanceDown();
    }

   }

   private void setMode(String somemode)
   {
     if(somemode.equalsIgnoreCase("STAY"))
      mode = WumpusMode.STAY;
     else if(somemode.equalsIgnoreCase("SPIN"))
      mode = WumpusMode.SPIN;
     else if(somemode.equalsIgnoreCase("WALK"))
      mode = WumpusMode.WALK;
     else if(somemode.equalsIgnoreCase("GOTO_GOLD"))
      mode = WumpusMode.GOTO_GOLD;
     else if(somemode.equalsIgnoreCase("GOTO_PIT"))
      mode = WumpusMode.GOTO_PIT;
     else
      mode = WumpusMode.RANDOM;
   }

}





