package wworld;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.JFrame;
import java.util.*;

/**
Class specifying the grapical component of the server side. The object
ProjectFrame has a complete graphical layout and display and update methods
for the graphical display.
@author Kruti Mehta
*/
public class projectFrame extends JFrame
{
   TextArea area1;
   TextArea area2;
   TextArea area3;
   TextArea area4;
   JPanel panel0;
   kPanel panel1;
   boolean gameStart = false;
   boolean gameEnd = false;
   TextField numTrialsText = new TextField();
   TextField currentTrialText = new TextField();
   TextField stepNumberText = new TextField();
   TextField maxStepsText = new TextField();
   private int worldsize;

   /**
   Sets the Wumpus world size.
   */
   public void setWorldSize(int s)
   {
    worldsize = s;
   }

   /**
   Constructor. The constructor also specifies the look and feel
   for the graphical display of the world
   */
   public projectFrame()
   {
    //this.getContentPane().setLayout(new GridLayout(2, 1));
    setTitle("Wumpus World Simulator Game Server");
    setSize(700, 700);

    GridBagLayout gbl = new GridBagLayout();
    this.getContentPane().setLayout(gbl);

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.gridx = 1;
    gbc.gridy = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 1;
    gbc.weighty = 0.001;

    panel0 = new JPanel();
    GridBagLayout gbl0 = new GridBagLayout();
    panel0.setLayout(gbl0);
    GridBagConstraints gbc0 = new GridBagConstraints();
    gbc0.fill = GridBagConstraints.BOTH;
    gbc0.weightx = 0.1;
    panel0.add(new Button("Start"),gbc0);
    panel0.add(new Button("End"),gbc0);
    gbc0.weightx = 0.1;
    panel0.add(new Label("Current Trial: ",Label.RIGHT),gbc0);
    panel0.add(currentTrialText,gbc0);
    panel0.add(new Label("Total Trials: ",Label.RIGHT),gbc0);
    panel0.add(numTrialsText,gbc0);
    panel0.add(new Label("Current Step: ",Label.RIGHT),gbc0);
    panel0.add(stepNumberText,gbc0);
    panel0.add(new Label("Max Steps: ",Label.RIGHT),gbc0);
    panel0.add(maxStepsText,gbc0);

    this.getContentPane().add(panel0,gbc);

    gbc.gridx = 1;
    gbc.gridy = 2;

    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weighty = 0.999;
    panel1 = new kPanel();

    this.getContentPane().add(panel1,gbc);
   }

   /**
   Displays the world and objects within the world.
   */
   public void displayWorld(Vector[][] grid, Vector agentVector)
   {
    panel1.populateGrid(grid, worldsize, agentVector);
    panel1.paintSquare();
   }

   /**
   Executes the start of the game and end of the game actions from
   the server GUI.
   */
   public boolean action(Event evt, Object arg)
   {
    if(arg instanceof String)
    {
     String s = (String) arg;
     if(arg == "Start")
      gameStart = true;
     if(arg == "End")
     {
      EndDialog ed = new EndDialog(this);
      ed.show();
      gameEnd = ed.getGameEnd();
     }
    }
    return true;
   }


   /**
   Returns TRUE if the game start button has been pressed.
   */
   public boolean getGameStart()
   {
    return gameStart;
   }

   /**
   Returns TRUE if the gameEnd is TRUE.
   */
   public boolean getGameEnd()
   {
    return gameEnd;
   }

   /**
   Sets start of the game attribute gameStart.
   */
   public void setGameStart(boolean value)
   {
    gameStart = value;
   }

   /**
   Sets the end of the game attribute gameEnd.
   */
   public void setGameEnd(boolean value)
   {
    gameEnd = value;
   }

   /**
   Sets the current Trial number display on the server GUI.
   */
   public void setCurrentTrial(int currentTrial)
   {
    currentTrialText.setText(Integer.toString(currentTrial));
   }

   /**
   Sets the current step number display on the server GUI.
   */
   public void setCurrentStep(int stepNumber)
   {
    stepNumberText.setText(Integer.toString(stepNumber));
   }

   /**
   Sets total number of Trials display on the server GUI.
   */
   public void setNumTrials(int numTrials)
   {
    numTrialsText.setText(Integer.toString(numTrials));
   }

   /**
   Sets the maximum number of steps display on server GUI.
   */
   public void setMaxSteps(int maxSteps)
   {
    maxStepsText.setText(Integer.toString(maxSteps));
   }


}


/**
Class specifying the end of the game Dialog. This dialog comes up when
user presses "End Game" button on the server GUI.
@author Kruti Mehta
*/
class EndDialog extends Dialog
{
 boolean gameEnd = false;

 /**
 Constructor. The window has confirming dialog and Yes, No buttons.
 Yes results in game end and program exit.
 No results in continuation of the game.
 */
 public EndDialog(projectFrame parent)
 {
  super(parent,"End Game",true);

  Panel p1 = new Panel();
  p1.add(new Label("Are you sure you want to end this game?",Label.CENTER));
  add("Center",p1);

  Panel p2 = new Panel();
  p2.add(new Button("Yes"));
  p2.add(new Button("No"));
  add("South",p2);
  this.setSize(240,120);
 }

 /**
 Performs exiting action when End of Game option on the Dialog is selected.
 */
 public boolean action(Event evt, Object arg)
 {
  if(arg.equals("Yes"))
  {
   gameEnd = true;
  }
  else
   gameEnd = false;
  dispose();
  return true;
 }

 public boolean getGameEnd()
 {
  return gameEnd;
 }
}


/**
Class specifying the display of the wumpus world on the server GUI.
@author Kruti Mehta
*/
class kPanel extends JPanel
{
 boolean showpit;
 boolean showobstacle ;
 boolean showgold;
 boolean showwumpus;
 Vector[][] grid;
 private int size;
 Vector agents;

 /**
 Populates the wumpus world information in the display.
 */
 public void populateGrid(Vector[][] agrid, int worldsize,Vector VecOfAgents)
 {
  grid = agrid;
  size = worldsize;
  agents = VecOfAgents;
 }

 /**
 Constructor.
 */
 public kPanel()
 {

  showpit = showobstacle = showgold = showwumpus = false;
  this.setBackground(Color.blue);
  this.setAutoscrolls(true);
 }

 /**
 Displays the wumpusworld on the server GUI.
 */
 public void paintComponent(Graphics g)
 {
   super.paintComponent(g);
   Vector agentsToBeDisplayed = new Vector();
   int agentCount= 0;

   g.setColor(Color.yellow);

   for(int i = 1; i <=size+1; ++i)
    for(int j = 1; j <=size; ++j)
     {
      drawHorizontalGridLine(g, i,j);
      drawVerticalGridLine(g, i, j);
     }

    for(int k = 0; k < size; ++k)
      for(int m = 0; m < size; ++m)
       {
        agentsToBeDisplayed.removeAllElements();
        if(agents != null)
        {
         for(int n = 0; n < agents.size(); ++n)
          {
            Agent agentn = (Agent)agents.elementAt(n);
            boolean kandx;
            boolean mandy;
            kandx = ( k==agentn.getLocation().getPtX());
            mandy = (m == agentn.getLocation().getPtY());
            if(kandx && mandy)
            {
             agentsToBeDisplayed.add(agentn);
            }
          }
         }
         displaySquare(g, grid[k][m], k, m, agentsToBeDisplayed);
       }

 }


 /**
 Displays each square for the wumpus world. The square displayed is with
 any objects within it. Each square of the wumpus world is displayed by
 calling this method from paintcomponent.
 */
 public void displaySquare(Graphics g, Object obj, int row, int col, Vector agentsToShow)
 {
    Vector vec = (Vector)obj;

    //System.out.println("INside DisplaySquare....." + row + " ..." + col);
    boolean showpit = false;
    boolean showgold = false;
    boolean showobstacle = false;
    boolean showwumpus = false;
    boolean showagent = false;

    Image img;

    for(int counter = 0; counter < vec.size(); ++counter)
    {
     if(vec.elementAt(counter)instanceof Pit && showpit ==false)
     {
      showpit = true;
      img = Toolkit.getDefaultToolkit().getImage("pit.gif");
      g.drawImage(img, (row*66+34),((size-col-1)*66+2), 32, 32, this);
            //upper right corner of square
     }

     else if(vec.elementAt(counter)instanceof Gold && showgold ==false)
      {
       showgold = true;
       img = Toolkit.getDefaultToolkit().getImage("pot_o_gold.gif");
       g.drawImage(img, row*66+34, (size - col-1)*66+34, 32, 32, this);
            //lower right corner of square
      }
     else if(vec.elementAt(counter)instanceof Obstacle && showobstacle ==false)
      {
       showobstacle = true;
       img = Toolkit.getDefaultToolkit().getImage("Obstacle.gif");
       g.drawImage(img, row*66+2, (size - col-1)*66+2, 64 , 64, this);
             //lower right corner of square
      }
     else if(vec.elementAt(counter)instanceof Wumpus && showwumpus ==false)
      {
       showwumpus = true;
       Wumpus thisWumpus = (Wumpus)vec.elementAt(counter);
       if(thisWumpus.getOrientation() == Directions.DOWN)
        img = Toolkit.getDefaultToolkit().getImage("wumpusFront.gif");
       else if(thisWumpus.getOrientation() == Directions.LEFT)
        img = Toolkit.getDefaultToolkit().getImage("wumpusLeft.gif");
       else if(thisWumpus.getOrientation() == Directions.RIGHT)
        img = Toolkit.getDefaultToolkit().getImage("wumpusRight.gif");
       else
        img = Toolkit.getDefaultToolkit().getImage("wumpusBack.gif");

       g.drawImage(img, row*66+2, (size - col-1)*66+2, 32, 32, this);
            //upper left corner of square
      }
    }

    for(int count = 0; count < agentsToShow.size(); ++count)
    {
     if(count ==4)       //Max 4 agents displayed.
      break;
     else 
     {
      Agent agent = (Agent)agentsToShow.elementAt(count);
      int orient = agent.getOrientation();


      int agentNumber = agent.getActorId();
      String agentIdString = Integer.toString(agentNumber);
      String agentName = "agent" + agentIdString;
      String imgString = "";
      if(orient == Directions.DOWN)
      {
        imgString = agentName + "Down.gif";
        img = Toolkit.getDefaultToolkit().getImage(imgString);
      }
      else if(orient ==Directions.UP)
      {
        imgString = agentName + "Up.gif";
        img = Toolkit.getDefaultToolkit().getImage(imgString);
      }
      else if(orient == Directions.LEFT)
      {
        imgString = agentName + "Left.gif";
        img = Toolkit.getDefaultToolkit().getImage(imgString);
      }
      else if(orient == Directions.RIGHT)
      {
        imgString = agentName + "Right.gif";
        img = Toolkit.getDefaultToolkit().getImage(imgString);
      }
      else
        img = Toolkit.getDefaultToolkit().getImage("orangeUp.gif");

     int xOff = 0;
     int yOff = 0;

     if(count == 1)
      {
       xOff = 16;
       yOff = 16;
      }
     else if(count == 2)
       xOff = 16;
     else if(count == 3)
       yOff = 16;

     g.drawImage(img, row*66+2+xOff, (size - col-1)*66+34+yOff, 15, 15,this);

     }
    }
  }

 /**
 Draws the horizonal lines of the grid display on the server GUI.
 */
 public void drawHorizontalGridLine(Graphics g, int row, int column)
 {
  int startx = 66*(column-1)+1;
  int starty = 66*(row-1)+1;
  int height = 2;
  int width = 66;
  g.fillRect(startx, starty, width, height);
 }

 /**
 Draws vertical lines of the grid on the server GUI.
 */
 public void drawVerticalGridLine(Graphics g, int row,int column)
 {
  int startx = 66*(row-1)+1;
  int starty = 66*(column-1)+1;
  int height = 66;
  int width = 2;
  g.fillRect(startx, starty, width, height);
 }

 public void paintSquare()
 {
  repaint();
 }

}



